subroutine PTfcn(input, output, N)

!	This subroutine uses one x(i) as input and creates f=abs(Pset-Pcalc)
!	as output.  N is set to one (i.e. only one variable is being changed and one equation needed).  
!	This subroutine 
!
!
!	This code needs the following subroutines
!		PTthermo_props.f90				dispersion.f90
!		ghs_derivatives.f90				hard_chain_eos.f90
!		hbond_thermo.f90				Q_sub.f90
!		zeta_s.f90						hs_cs.f90
!						
!	This routine also uses the common bloc and converts it to something thermo_props likes
!		Because of this, the size of arrays are temporarily set to 25 (and max to 8) and then 
!		converted accordingly to the number of components present for use in thermo_props.  
!		This is the only way to put these in common blocs.  A pointer can be used instead, but the 
!		program will have to be changed to accommodate the pointers (including thermo_props).
!
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			8/04/99		P.D. Ting				inception

use essentials 
implicit none

!	Declaring calling variables
real (KIND=double), intent(in) :: input			! input values at which the functions are evaluated
real (KIND=double), intent(out) :: output		! The computed function value at input 
integer :: N									! number of equations and unknowns 

!	Declaring thermo_props scalable variables

real (KIND=double), dimension(ncp) :: m
real (KIND=double), dimension(ncp,ncp) :: u0
real (KIND=double), dimension(ncp) :: e_ii
real (KIND=double), dimension(ncp,ncp,max,max) :: kappa
real (KIND=double), dimension(ncp,ncp) :: sig
real (KIND=double), dimension(ncp) :: v00
real (KIND=double), dimension(ncp,ncp) :: kij
real (KIND=double), dimension(ncp) :: x,xd,xq, Q, mu  ! Mole fraction, polar fraction and quadrupolar fraction
! quadrupolar moment, dipole moment
real (KIND=double), dimension(ncp,ncp,max,max,2) :: fa
real (KIND=double), dimension(ncp,ncp,max,max,1) :: fb
real (KIND=double) :: Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, Pcalc, Gtotal
real (KIND=double), dimension(ncp) :: chpt, fugacity, chpt_res

integer, dimension(ncp) :: N_sites

!	Declaring local variables
integer :: i,j,a,b											! counters
real (KIND=double) :: sum, luck, pdiff, rhox
real (KIND=double) :: sitos1(ncp), dhs(ncp), sitos1c(ncp)
real (KIND=double) :: zeta3
real (KIND=double), parameter :: pi = 3.141592654d0
real (KIND=double) :: pi_over6 = pi / 6.0D0


external dimension_convert, PTthermo_props

!	Setting up parameters.  We will use the integer variables as is from module but rescale the 
!	array size of all arrays from a rank of 25 to one of ncp.

call dimension_convert(m, u0, e_ii, kappa, sig, v00, kij, x, fa, &
		&fb, N_sites,q,mu,xq,xd)

output = 0.0d0
zeta3 = 0.0d0

forall (i=1:ncp)
  dhs(i) = sig(i,i) * (1.0D0 - 0.12D0 * exp(-3.0D0*u0(i,i) / TK))
  sitos1(i) = dhs(i) / sig(1,1)
end forall

forall (i=1:ncp)
  sitos1c(i) = sitos1(i) * sitos1(i) * sitos1(i)
end forall

!	Calculating zeta and derivative of zeta wrt temperature

do i = 1, ncp
  zeta3 = x(i) * m(i) * sitos1c(i) + zeta3
end do

zeta3 = pi_over6 * input * zeta3

N = 1
rhox = input 

if (rhox .lt. 0.0d0) then 
  output = 10000000.0
end if	

if ( zeta3 .ge. 0.999d0) then
  output = 10000000.0
end if
	
if (output .lt. 100.0d0) then

  call PTthermo_props(ncp, max, status11, TK, rhox, m, x, u0, e_ii, kij, &
			& N_sites, fa, fb, kappa, sig, D, v00, Prsm, Ursm, &
			& Arsm, Grsm, Srsm, Hrsm, chpt, Pcalc, fugacity, chpt_res, Gtotal,PC_dispersion,&
			& analytic_flag, polar_flag, Q, mu, xd, xq) 

  Pdiff = abs(Pcalc - Pset) * 100.0
  output = Pdiff
end if
end subroutine 

