Subroutine driver_phasecalc(pbb, fugacity_z, phase, limit, counter, phaseX, phaseY, &
						& rhoxphaseX, rhoxphaseY)

!	This subroutine is the driver for the equilibrium phase calculation using Rachford-Rice
!	formalism.  

!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			3/31/00		P.D. Ting				inception
!

use essentials
implicit none

!	Input/output arguments
real (KIND=double), dimension(ncp),intent(in) :: fugacity_z		! The fugacity of the feed
real (KIND=double), dimension(ncp),intent(in) :: pbb			! Composition of an 'unstable' point
real (KIND=double), intent(out) :: limit
integer, intent(out) :: counter
integer :: phase

!	Internal working variable
real (KIND=double) :: rhox_pbb				! The density of the 'unstable' point
real (KIND=double), dimension(ncp) :: x_passaround
integer :: itmax, flag3, flag4, i, N
real (KIND=double) :: alpha_guess, errel, alpha_result, fnorm, sum, sum2
real (KIND=double), dimension(ncp) :: phaseX, phaseY, fugacityX, fugacityY, pseudoX
real (KIND=double) :: rhoxphaseX, rhoxphaseY


!	Thermoprops required variables
real (KIND=double), dimension(ncp) :: m
real (KIND=double), dimension(ncp,ncp) :: u0
real (KIND=double), dimension(ncp) :: e_ii
real (KIND=double), dimension(ncp,ncp,max,max) :: kappa
real (KIND=double), dimension(ncp,ncp) :: sig
real (KIND=double), dimension(ncp) :: v00
real (KIND=double), dimension(ncp,ncp) :: kij
real (KIND=double), dimension(ncp) :: x, xd,xq,q,mu
real (KIND=double), dimension(ncp,ncp,max,max,2) :: fa
real (KIND=double), dimension(ncp,ncp,max,max,1) :: fb
integer, dimension(ncp) :: N_sites
real (KIND=double) :: Prsm						! total residual P
real (KIND=double) :: Pcalc					! calculated Pressure in bars
real (KIND=double) :: Ursm						! total residual U
real (KIND=double) :: Arsm						! total residual A
real (KIND=double) :: Grsm						! total residual G
real (KIND=double) :: Srsm						! total residual S
real (KIND=double) :: Hrsm						! total residual H
real (KIND=double) :: Gtotal					! Total Gibbs free energy of mixture
real (KIND=double), dimension(ncp) :: chpt		! total chemical potential
real (KIND=double), dimension(ncp) :: chpt_res	! residual total chemical potential
real (KIND=double), dimension(ncp) :: fugacity	! natural log of fugacity coefficient

external partition_calc, dneqnf, dimension_convert, liquid_or_vapor, PTthermo_props, &
		& partition_setup

do i=1,ncp
  x_passaround(i) = x_tmp(i)
  x_tmp(i) = pbb(i)
end do

call dimension_convert(m, u0, e_ii, kappa, sig, v00, kij, x, fa, &
		&fb, N_sites,q,mu,xq,xd)

call liquid_or_vapor(rhox_pbb, pbb)

call PTthermo_props(ncp, max, status11, TK, rhox_pbb, m, pbb, u0, e_ii, kij, N_sites, fa, &
				& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt, &
				& Pcalc, fugacity, chpt_res, Gtotal, PC_dispersion, analytic_flag,&
				& polar_flag, Q, mu, xd, xq) 

do i = 1, ncp
  x_tmp(i) = x_passaround(i)
end do

!	partition_setup sets up the initial partition values and stores it in the module
call partition_setup(pseudoX, fugacity, phase)


!	Initializing IMSL and other parameters
alpha_guess = 0.50d0
!MAXFCN = 20000
N = 1
errel = 1.0d-12
itmax = 1000000
counter = 0		
limit = 100.0


! now use successive substitution to find composition of next phase.		
do while ((limit .gt. 1.0d-6) .and. (counter .lt. 6000)) 

 
  call DNEQNF(partition_calc, errel, N, itmax, alpha_guess, alpha_result,&
			&	fnorm)

  sum = 0.0d0
  sum2 = 0.0d0

  do i = 1, ncp
	phaseX(i) = z(i)/(1.0d0 + (partition(i)-1.0d0)*alpha_result)
	phaseY(i) = z(i)*partition(i)/(1.0d0+(partition(i)-1.0d0)*alpha_result)
	sum = sum + phaseX(i)
	sum2 = sum2 + phaseY(i)
  end do

  if ( (sum .gt. 1.0001) .or. (sum .lt. 0.9999) ) then
	forall (i=1:ncp)
      phaseX(i) = phaseX(i) / sum
	end forall
  end if
  if ( (sum2 .gt. 1.0001) .or. (sum2 .lt. 0.9999) ) then
	forall (i=1:ncp)
      phaseY(i) = phaseY(i) / sum2
	end forall
  end if 
			
!	Now i need to find the fugacity of these guys 
  forall (i=1:ncp)
	x_passaround(i) = x_tmp(i)
    x_tmp(i) = phaseX(i)	
  end forall

  call liquid_or_vapor(rhoxphaseX, phaseX)	

  call PTthermo_props(ncp, max, status11, TK, rhoxphaseX, m, phaseX, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt, &
						& Pcalc, fugacityX, chpt_res, Gtotal, pc_dispersion, analytic_flag,&
						& polar_flag, Q, mu, xd, xq) 
  forall (i=1:ncp)
    x_tmp(i) = phaseY(i)
  end forall

  call liquid_or_vapor(rhoxphaseY, phaseY)

  call PTthermo_props(ncp, max, status11, TK, rhoxphaseY, m, phaseY, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt, &
						& Pcalc, fugacityY, chpt_res, Gtotal, pc_dispersion, analytic_flag,&
						& polar_flag, Q, mu, xd, xq)

!  Check to see if any of these phase become nonexistent			

  sum = 0.0d0
  sum2 = 0.0d0
  flag3 = 1
  flag4 = 1
  do i = 1, ncp
	if (phaseY(i) .lt. 1.0d-25) then
	  flag3 = 0	
	end if
	if (phaseX(i) .lt. 1.0d-25) then
	  flag4 = 0
	end if
  end do

!	two_phase_flash updates the partition and calculates new phaseX and phaseYs
			
  call two_phase_flash(flag3, flag4, phaseY, phaseX, fugacityY, fugacityX, limit)

  counter = counter + 1
  call PTthermo_props(ncp, max, status11, TK, rhoxphaseX, m, phaseX, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt, &
						& Pcalc, fugacityX, chpt_res, Gtotal, pc_dispersion, analytic_flag,&
						& polar_flag, Q, mu, xd, xq) 

  ! write(6,*) 'Ursm', Ursm
end do 

END SUBROUTINE

