subroutine funcv(den,f,N)
use essentials

implicit none
!
!*********************************************************
!Subroutine to calculate the difference between chemical *
!potentials and pressures of two phases for a constant	 * 
!T-x flash (bubble pressure calculation.)	             *  
!                                                        * 
!*********************************************************

!Declaring variables
integer, intent(in) :: N	! Number of equations=ncp+1
integer :: i,j,k,l
!

real(KIND=double), intent(in), dimension(N) :: den ! Densities of two phases and composition of second phase
real(KIND=double), intent(out), dimension(N) :: f   ! Function values 
real(KIND=double) :: rhoxL	! Reduced density of phase 1
real(KIND=double) :: rhoxV  ! Reduced density of phase 2
real (KIND=double) :: sumv  ! Temporary accumulator 
!
!	Declaring parameters
real (KIND=double) :: AVOG = 0.602252d0				! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451					! cm3 bar/mol /k
!
!	Thermoprops required variables
real (KIND=double), dimension(ncp) :: m, Q, mu, xq, xd
real (KIND=double), dimension(ncp,ncp) :: u0
real (KIND=double), dimension(ncp) :: e_ii
real (KIND=double), dimension(ncp,ncp,max,max) :: kappa
real (KIND=double), dimension(ncp,ncp) :: sig
real (KIND=double), dimension(ncp) :: v00
real (KIND=double), dimension(ncp,ncp) :: kij
real (KIND=double), dimension(ncp,ncp,max,max,2) :: fa
real (KIND=double), dimension(ncp,ncp,max,max,1) :: fb
integer, dimension(ncp) :: N_sites
real (KIND=double) :: Prsm						! total residual P 
real (KIND=double) :: Ursm						! total residual U
real (KIND=double) :: Arsm						! total residual A
real (KIND=double) :: Grsm						! total residual G
real (KIND=double) :: Srsm						! total residual S
real (KIND=double) :: Hrsm						! total residual H
real (KIND=double) :: GtotalL, GtotalV, PLold, fsum1, fsum2,fsum3				! Total Gibbs free energy of mixture
real (KIND=double), dimension(ncp) :: chptL, chptV	! total chemical potential for liq. and vapor
real (KIND=double), dimension(ncp) :: chpt_res	! residual total chemical potential
real (KIND=double), dimension(ncp) :: fugacity	! natural log of fugacity coefficient
real (kind=double), dimension(N)   :: f_err
logical :: advance
! Place an error checking flag here to skip the thrmoprops if anything in den is negative
100 format ('liq. Chem. Pot. for comp. ',I2' is ', E14.8,'')
110 format ('vap. Chem. Pot. for comp. ',I2' is ', E14.8,'')
120 format (' at the end of funcv, den',I2,'is',f12.6, '')
130 format (' at the end of funcv, f',I2,'is',E12.6, '')
140 format (' at the begin of funcv, den',I2,'is',f12.6, '')
advance = .true.
! This checks to see if the density guesses are ok. 

do j=1,N
	!write(6,140),j,den(j)
	if (den(j).lt.0.0) then
		advance =.false.
		
		end if
!	end if
!
end do
		

if (advance) then 
	forall (i=1:ncp)
		x_tmp(i)=xliq(i)
	end forall

	call dimension_convert(m, u0, e_ii, kappa, sig, v00, kij, xliq, fa, &
		&fb, N_sites,q,mu,xq,xd)
!
	rhoxL=den(1)
!
	purefluid=.FALSE.
	
	do i=1,ncp
		if(xliq(i).eq.1.0) then
			purefluid=.TRUE.
			ipure=i
		end if
	end do 
	
	write(6,*) 'the liquid reduced density before the TP liquid call is',rhoxl
	write(6,*) 'Liquid mole fraction, component 1 is',xliq(1)
	write(6,*) 'Liquid mole fraction, component 2 is',xliq(2)
	!write(6,*) 'Liquid pressure is',pl

	! This is a check to see if the liqid phase pressure is actually moving or not. 	
	
	
	call PTthermo_props(ncp, max, status11, TK, rhoxL, m, xliq, u0, e_ii, kij, N_sites, fa, &
				& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chptL, &
				& PL, fugacity, chpt_res, GtotalL,pc_dispersion, analytic_flag,&
				& polar_flag, Q, mu, xd, xq) 

!	
	!write(6,*) 'the liquid reduced density after the tp liquid call is',rhoxl

	rhoxV=den(2)
	
!
	if(purefluid) then

		forall (i=1:ncp)
			xvap(i)=0.d0
		end forall
		! this sets the pure fluid vapor molf to 1.0
		xvap(ipure)=1.d0

		else  
			sumv=0.0d0

		do i=1,ncp-1
			xvap(i)=den(2+i)
			sumv=sumv+xvap(i)
		end do 
			xvap(ncp)=1.0d0-sumv
	end if


	forall (i=1:ncp)
		x_tmp(i)=xvap(i)
	end forall

	call PTthermo_props(ncp, max, status11, TK, rhoxV, m, xvap, u0, e_ii, kij, N_sites, fa, &
				& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chptV, &
				& PV, fugacity, chpt_res, GtotalV, pc_dispersion, analytic_flag,&
				& polar_flag, Q, mu, xd, xq)  

	
	! Now calculate the difference in chemical potentials and difference in pressures
	!
	if (purefluid) then

		f(1)=GtotalL-GtotalV
		f(2)=(PL-PV)*100.00 !/(R*TK) was removed to try and get things to converge a bit better

		forall (i=3:N)
			f(i)=0.0d0
		end forall

		else ! this is for mixtures
			
			do i=1,ncp
				f(i)=chptL(i)-chptV(i)
				write(6,fmt = 100) i,chptL(i)
				write(6,fmt = 110) i,chptV(i)
			end do		

			f(N)=(PL-PV)! *100.00 !/(R*TK) !was removed to try and get things to converge a bit better
			
	end if ! pure fluid endif 

else ! advance loop
	write(6,*) 'entering the consequence loop'

	
	f(N)=(PL-PV)
			
	do i=1,ncp
		if (den(i).lt.0) then
			f(i) = 10000 
			write(6,*) 'a density was less than zero'
		end if
		
		if (den(i).lt.-0.01) then
			! f(i) = 6000
			write(6,*) 'a density was less than -0.3'
		end if
	
	end do

end if ! advance if

do i =1,N
	write(6, 120) i,den(i) 
	write(6, 130) i,f(i)
end do


write(6,*) 'At the end of funcv, liquid pressure is',PL
write(6,*) 'At the end of funcv, vapor pressure is', PV
!write(6,*) 'liquid density is',rhoxl
!write(6,*) 'vapor density is',rhoxV
!write(6,*) ' ' 
! pause
end subroutine