function kij_mini_uvmif(kij_guess)

!	This function calculates the deviation between experimental data and data calculated using
!	kij_guess as the 2 component parameter.

use essentials
implicit none

!	Internal working variables
integer :: i,j, iersvr, ipact, isact
integer :: phase, counter
real (KIND=double) :: sum, kij_mini_uvmif
real (KIND=double) :: x_p(2), y_p(2)
real (KIND=double) :: f_value(ndp_exp*2)
real (KIND=double) :: kij_guess
real (KIND=double), dimension(ncp) :: x, pbb, fugacity_z, phaseX, phaseY
real (KIND=double), dimension(7) :: rparam2
integer, dimension(7) :: iparam2
real (KIND=double) :: limit, rhox, rhoxphasex, rhoxphasey, ybb, fvalue_uminf
real (KIND=double) :: temp


kij_tmp(1,1) = 0.0d0
kij_tmp(2,2) = 0.0d0
kij_tmp(1,2) = kij_guess
kij_tmp(2,1) = kij_guess

iersvr = 0
ipact = 0
isact= 0 
call DU4INF(iparam2, rparam2)
iparam2(3) = 100000
iparam2(4) = 100000
iparam2(6) = 0



do i = 1, ndp_exp
  x_tmp(2) = (x_exp(2,i)+y_exp(2,i)) / 2
  x_tmp(1) = 1.0d0 - x_p(2)

  Pset = Pset_exp(i)
  TK = TK_exp(i) 
    
!	apply SAFT to the points of interest
  forall (j=1:ncp)
    x(j) = x_tmp(j)
	z(j) = x_tmp(j)   
  end forall

  call liquid_or_vapor(rhox, x)
  rhox_tmp = rhox

  call driver_michelsen(rhox, pbb, ybb, fvalue_uminf, iparam2, rparam2, fugacity_z)
  if (fvalue_uminf.gt.-1.0d-17) then
    call driver_stability2(ybb,pbb, iparam2, rparam2)
  end if

  if (ybb.le.-1.0d-17) then
    do phase=1,2

      call driver_phasecalc(pbb, fugacity_z, phase, limit, counter, phaseX, phaseY, &
				&	rhoxphaseX, rhoxphaseY)

	  if ((limit.lt.1.0d-6).and.(counter.lt.6000)) EXIT
	  if ((phase.eq.2).and.(limit.gt.1.0d-6).and.(counter.eq.6000)) write(6,*) 'no solution'
    end do

    if (rhoxphaseY.gt.rhoxphaseX) then
	  temp = rhoxphaseX
	  rhoxphaseX = rhoxphaseY
	  rhoxphaseY = temp

	  temp = phaseX(1)
	  phaseX(1) = phaseY(1)
	  phaseY(1) = temp

	  phaseX(2) = 1.0d0 - phaseX(1)
	  phaseY(2) = 1.0d0 - phaseY(1)
	end if

  else 
    write(6,*) 'The internally generated species composition, temperature,' 
    write(6,*) 'and pressure seems to be stable as a one phase system.'
    write(6,*) 'Program halted.'
  end if

  x_p(1) = phaseX(1)
  x_p(2) = phaseX(2)
  y_p(1) = phaseY(1)
  y_p(2) = phaseY(2)

!	The objective function below is the absolute value of the difference between the calculated and observed values

  f_value(i) = abs( ( x_p(2) - x_exp(2,i) )/min( (1.0d0-x_exp(2,i) ), x_exp(2,i) ) )
  f_value(i+ndp_exp) = abs( (y_p(2) - y_exp(2,i) ) /min( (1.0d0-y_exp(2,i) ), y_exp(2,i) ) )

end do

sum = 0.0d0
do i = 1, 2*ndp_exp 
  sum = sum + f_value(i)
end do

kij_mini_uvmif = sum

end function











