SUBROUTINE liquid_or_vapor(rhox,x)

!	This subroutine checks whether a given feed (T,P, xi) is liquid-like or 
!	vapor-like.  It will then return the appropriate density.

!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			3/31/00		P.D. Ting				inception
!

use essentials
implicit none


!	Internal variables
integer :: phase, i								! liquid or vapor-like phase
real(KIND=DOUBLE) :: sum						! generic counter
real(KIND=double) :: reducedrho					! densities
real(KIND=double) :: rhox_guess, rhox_result
real(KIND=double), dimension(2) :: Gtotal_compare		! temporary storage for Gibbs free energy
real(KIND=double), dimension(2) :: rhox_compare			! temporary storage for rhox
real(KIND=double), dimension(2) :: Pcalc_compare		! temporary storage for P
real(KIND=double), intent(out) :: rhox					! number density 

!	IMSL routine variables
real(KIND=double) :: errel, fnorm
integer :: iersvr, ipact, isact, N, itmax		! IMSL routine usage

!	Thermoprops variables
real (KIND=double), dimension(ncp) :: m
real (KIND=double), dimension(ncp,ncp) :: u0
real (KIND=double), dimension(ncp) :: e_ii
real (KIND=double), dimension(ncp,ncp,max,max) :: kappa
real (KIND=double), dimension(ncp,ncp) :: sig
real (KIND=double), dimension(ncp) :: v00, Q, xd,xq, mu
real (KIND=double), dimension(ncp,ncp) :: kij
real (KIND=double), intent(in), dimension(ncp) :: x
real (KIND=double), dimension(ncp,ncp,max,max,2) :: fa
real (KIND=double), dimension(ncp,ncp,max,max,1) :: fb
real (KIND=double) :: Prsm, Arsm, Srsm, Ursm, Grsm, Hrsm, Pcalc, Gtotal
real (KIND=double), dimension(ncp) :: chpt, fugacity, chpt_res
integer, dimension(ncp) :: N_sites



!	Internal parameters
real (KIND=double) :: AVOG = 0.602252d0 			! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451					! cm3 bar/mol /k
real (KIND=double) :: pi = 3.141592654d0
real (KIND=double) :: tau = 0.7404804897

external PTFCN, dimension_convert, PTThermo_props

!	Parameters used in DNEQNF
iersvr = 0
ipact = 0
isact = 0
N = 1
errel = 1.0d-12
itmax = 500000


do phase = 1,2

  sum = 0.0d0
  do i = 1, ncp
	sum = x(i) * m_tmp(i) *sig_tmp(i,i)**3 + sum
  end do

  if (phase .eq. 2) then					! treat as vapor
	rhox = Pset / TK / R * AVOG * sig_tmp(1,1)**3 /100.0d0
	reducedrho = rhox * sum / sig_tmp(1,1)**3
	if (reducedrho .gt. 0.99) then
  	  rhox = 0.10d0 / sum * sig_tmp(1,1)**3
	end if

  else 
	rhox = 0.90d0 / sum * sig_tmp(1,1)**3		! treat as liquid
  end if

  rhox_tmp = rhox 
  rhox_guess = rhox
		
  forall (i=1:ncp)
    x_tmp(i) = x(i)
  end forall

! problem occurred here!!!!!


  call erset(iersvr, ipact, isact)
  call DNEQNF(PTfcn, errel, N, itmax, rhox_guess, rhox_result, fnorm)
 

  call dimension_convert(m, u0, e_ii, kappa, sig, v00, kij, x, fa, &
		&	fb, N_sites,q,mu,xq,xd)
	

  call PTthermo_props(ncp, max, status11, TK, rhox_result, m, x, u0, e_ii, &
	&	kij, N_sites, fa, fb, kappa, sig, D, v00, Prsm, &
	&	Ursm, Arsm, Grsm, Srsm, Hrsm, chpt, Pcalc, fugacity, chpt_res, Gtotal, PC_dispersion,&
	&   analytic_flag, polar_flag, Q, mu, xd, xq)


  Gtotal_compare(phase) = Gtotal
  rhox_compare(phase) = rhox_result
  Pcalc_compare(phase) = Pcalc

end do



!	Determine whether the liquid or vapor (one) phase is more stable and use the stable
!	phase as the starting point for further calculations.

if ( (Pcalc_compare(1) .gt. (Pset+1.0d0) ) .or. (Pcalc_compare(1) .lt. (Pset - 1.0d0)) ) then
	 Gtotal_compare(1) = 1.0d20
end if

if ( (Pcalc_compare(2) .gt. (Pset+1.0d0) ) .or. (Pcalc_compare(2) .lt. (Pset - 1.0d0)) ) then
	 Gtotal_compare(2) = 1.0d20
end if


if ( Gtotal_compare(1) .lt. Gtotal_compare(2) ) then
		Gtotal = Gtotal_compare(1)
		rhox = rhox_compare(1)
	else
		Gtotal = Gtotal_compare(2)
		rhox = rhox_compare(2)
end if


END SUBROUTINE	
