
Subroutine user_interface(pick)


!	This is the front end/user interface for SAFT.  It is a series of 
!	menu options that lets you specify the system you are interested in and 
!	pulls out the relevant information from the database.
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			3/31/00		P.D. Ting				inception
!			10/15/04    M. Yarrison				Added text file input, polar and PC_SAFT
!												capabilities

use essentials
implicit none


!	Declaring parameters
integer :: switch, flag,i, j, k, l			!  temporary switching flags
integer :: N_sites								!  number of association sites 
integer :: pick, pick2, pick3

real (KIND=double) :: v00, m, u0, e_ii, kappa, epsilon, mw	!molecular parameters
real (KIND=double) :: sum									! temporary calculation variable

!	Declaring local parameters
real (KIND=double) :: AVOG = 0.602252d0 			! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451					! cm3 bar/mol /k
real (KIND=double) :: pi = 3.141592654d0
real (KIND=double) :: tau = 0.7404804897


3000 format ('component(', I2, ')')


write(6,*) '===================================================='
write(6,*) '=       WELCOME TO THE SAFT USER INTERFACE         ='
write(6,*) '=                                                  ='
write(6,*) '=                                                  ='
write(6,*) '=  This is where you will input all relevant info. ='
write(6,*) '=  for SAFT to process.                            ='
write(6,*) '=                                                  ='
write(6,*) '=  Program written by:                             =' 
write(6,*) '=           Matt Yarrison                          ='
write(6,*) '=           P. David Ting                          ='
write(6,*) '=           Prasanna K. Jog                        ='
write(6,*) '=           Jorg Blaesing                          ='
write(6,*) '=           Walter G. Chapman                      ='  
write(6,*) '=                                                  ='
write(6,*) '=  version 0.76                                    ='
write(6,*) '=  (C) 2000,2004 by above authors                  ='
write(6,*) '===================================================='


write(6,*) 'Please input the number of components in your system'
read(5,*) ncp

write(6,*) ' '

write(6,*) ' '
write(6,*) ' '
write(6,*) '==========================================================================='
write(6,*) '=                                                                         ='
write(6,*) '=     Please specify one of the listed options:                           ='
write(6,*) '=     (1) use fixed x (at constant T) to calculate the phase diagram      ='
write(6,*) '=     (2) fit SAFT kij parameters using known isothermal exp. data        ='
write(6,*) '=                     (for 2 component systems only)                      ='
write(6,*) '=      NOTE: This option is SLOW. best to get a good estimate of kij      ='
write(6,*) '=            First and adjust this in the main_driver.                    ='
write(6,*) '=            Use <10 data points for fitting.                             ='
write(6,*) '=     (3) RESERVED FOR LATER USE                                          ='
write(6,*) '=                                                                         ='
write(6,*) '==========================================================================='
read(5,*) pick
 
write(6,*) ' '
write(6,*) ' '
write(6,*) ' '

write(6,*) '==========================================================================='
write(6,*) '=                                                                         ='
write(6,*) '=     Please specify one of the listed options:                           ='
write(6,*) '=     (1) Input parameters from a text file                               ='
write(6,*) '=     (2) Run from defined system in input_parameters.f90                 ='
write(6,*) '=     (3) RESERVED FOR LATER USE                                          ='                                                   
write(6,*) '=                                                                         ='
write(6,*) '==========================================================================='
read(5,*) pick2

write(6,*) ' '
write(6,*) ' '

if (pick2.eq.1) then
	write(6,*) '============================================================================'
	write(6,*) '=                                                                          ='
	write(6,*) '=   Please ensure that the parameters are in the file called in_param.txt  ='
	write(6,*) '=                                                                          ='
	write(6,*) '============================================================================'

	OPEN (Unit=12, FILE="in_param.txt", ACTION="read")

		read(12,*) status11
		read(12,*) PC_dispersion
		read(12,*) polar_flag
		read(12,*) analytic_flag
		
		do i=1, ncp
			read(12,*) v00_tmp(i)
			read(12,*) m_tmp(i)
			read(12,*) u0_tmp(i,i)
			read(12,*) mw_tmp(i)
			read(12,*) e_ii_tmp(i)
			read(12,*) N_sites_tmp(i)
!these are the polar parameters. Set them to zero if no polarity			
			read(12,*) q_tmp(i)
			read(12,*) xq_tmp(i)
			read(12,*) mu_tmp(i)
			read(12,*) xd_tmp(i)
		end do

			  write(6,*) ' '
				write(6,*) 'Enter the association epsilon (Well depth) interaction parameters below'
				do j = 1, ncp
					do i = 1, N_sites_tmp(j)
						do k = 1, ncp
							do l = 1, N_sites_tmp(k)
							!write(6,*) 'interaction between site',i,' on molecule', j,' and'
							!write(6,*) 'site',l, ' on molecule', k
							read(12,*) fa_tmp(j,k,i,l,2)
							end do
						end do
					end do
				end do

			write(6,*) ' '
			write(6,*) 'Enter the association kappa (Well width) interaction parameters below'
			do j = 1, ncp
				do i = 1, N_sites_tmp(j)
 					do k = 1, ncp
  						do l = 1, N_sites_tmp(k)
						!write(6,*) 'interaction between site',i,' on molecule', j, ' and'
  						!write(6,*) 'site',l, ' on molecule', k
						read(12,*) kappa_tmp(j,k,i,l)
						end do
					end do
				end do
			end do

  !write(6,*) ' '
  !write(6,*) 'Enter the interaction parameters Kij below'
  do i = 1, ncp
    do j = i+1, ncp
 	  !write(6,*) 'For i =',i, 'j=',j
	  read(12,*) kij_tmp(i,j)
	  kij_tmp(j,i) = kij_tmp(i,j)
    end do
  end do
						

			
		
		
		
	CLOSE ( unit=12,status="keep")
end if

if (pick2.eq.3) then

!	do i = 1, ncp
!		write(6,*) ' '
!		write(6,*) ' '
!		write(6,*) ' '
!		write(6,*) 'For Component ',i,' Please select the class of compound that best'
!		write(6,*) 'describes it'
!		write(6,*) ' '
!		write(6,*) '==============================================================================='
!		write(6,*) '=                                                                             ='
!		write(6,*) '= (1) Small non-associating Inorganics      (11)  Ketones                     ='
!		write(6,*) '= (2) n-Alkanes                             (12) Tertiary Amines              ='
!		write(6,*) '= (3) Polymers                              (13) Esters                       ='
!	    write(6,*) '= (4) n-Alkylcyclopentanes                  (14) Alkenes                      ='
!		write(6,*) '= (5) n-Alkylcyclohexanes                   (15) Chlorinated HxCys            ='
!		write(6,*) '= (6) Benzene Derivatives                   (16) RESERVED FOR LATER VERSIONS  ='
!		write(6,*) '= (7) Polynuclear Aromatics                 (17) RESERVED FOR LATER VERSIONS  ='
!		write(6,*) '= (8) Ethers                                (18) RESERVED FOR LATER VERSIONS  ='
!		write(6,*) '= (9) RESERVED FOR LATER VERSIONS                RESERVED FOR LATER VERSIONS  ='
!		write(6,*) '= (10)RESERVED FOR LATER VERSIONS           (20) RESERVED FOR LATER VERSIONS  ='
!		write(6,*) '=                                                                             ='
!		write(6,*) '==============================================================================='
!		write(6,*) ' '
!		write(6,*) 'Class of compound is:'
!		read(5,*) switch
	
!			if (switch.eq.1) then 
!				call dbase_list_nainorg(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!
!			else if (switch.eq.2) then
!				call dbase_list_nalkanes(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.3) then
!				call dbase_list_polymers(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.4) then
!				call dbase_list_nAlkylcyclopentanes(i, switch, v00, m, u0, e_ii, kappa, epsilon, &
!				 &	N_sites, mw)
!			else if (switch.eq.5) then
!				call dbase_list_nAlkylcyclohexanes(i, switch, v00, m, u0, e_ii, kappa, epsilon, &
!				&	N_sites, mw)
!			else if (switch.eq.6) then
!				call dbase_list_benzene(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.7) then
!				call dbase_list_polynucleararomatics(i, switch, v00, m, u0, e_ii, kappa, epsilon, &
!				&	 N_sites, mw)
!			else if (switch.eq.8) then
!				 call dbase_list_ethers(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!
!
!			else if (switch.eq.9) then
!				 call dbase_list_primaryamines(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.10) then
!				 call dbase_list_secondaryamines(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.11) then
!				 call dbase_list_ketones(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.12) then
!				 call dbase_list_tamines(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.13) then
!			else if (switch.eq.14) then
!				 call dbase_list_alkenes(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.15) then
!				call dbase_list_clhydroc(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.16) then
!				 call dbase_list_ainorg(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.17) then
!				 call dbase_list_alkanols(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.18) then
!				 call dbase_list_acids(i, switch, v00, m, u0, e_ii, kappa, epsilon, N_sites, mw)
!			else if (switch.eq.19) then
!			 write(6,*) 'Please enter the following info for component:',i
!			 write(6,*) 'v00:'
!			 read(5,*) v00
!			 write(6,*) 'm:'
!			 read(5,*) m
!			 write(6,*) 'u0:'
!			 read(5,*) u0
!			 write(6,*) 'e_ii:'
!			 read(5,*) e_ii
!			 write(6,*) 'kappa (set to 0 if nonapplicable):'
!	 		 read(5,*) kappa
!			 write(6,*) 'epsilon (set to 0 if nonapplicable):'
!			 read(5,*) epsilon
!			 write(6,*) 'N_sites:'
!			 read(5,*) N_Sites
!		 	 write(6,*) 'MW:'
!			 read(5,*) mw
!			 write(6,*) ' '

!			else if (switch.eq.20) then
!			 v00 = 16
!			 m = 35
!			 u0 = 350
!			 e_ii = 10
!			 kappa = 0.003149
!			 epsilon = 6185
!			 N_Sites = 2
!			 mw = 1099
!
!			end if

!	Putting all the component parameters in a matrix

!		m_tmp(i) = m
!		v00_tmp(i) = v00
!		u0_tmp(i,i) = u0
!		e_ii_tmp(i) = e_ii
!		N_sites_tmp(i) = N_sites
!		mw_tmp(i) = mw 
!	end do


!	max = 0
!	do i = 1, ncp
!		if (max .lt. N_sites_tmp(i)) then
!			max = N_sites_tmp(i)
!		end if
!	end do
!
!	if (max.eq.0) then
!		status11 = 0
!	else 
!		status11 = 1
!	end if

end if

if (pick2.eq.2) then
	write(6,*) ' Using pre-defined system in input_parameters'
	call input_parameters
end if


if (pick.eq.1) then

  write(6,*) 'Enter a desired (guess) pressure (in bars)'
  read(5,*) Pset
  write(6,*) ' '

  write(6,*) 'Enter the desired temperature (in Kelvin)'
  read(5,*) TK


  write(6,*) ' '
  write(6,*) 'Do you want to specify the mole fraction or the mass fraction of the system'
  write(6,*) 'you just entered?  (1) for mole fraction and (2) for mass fraction'
  read(5,*) polymer

  sum = 0.0d0 

  write(6,*) 'enter the initial mass/mole fractions below: '
  do i = 1, ncp
    if (i.eq. ncp) then
  	  write(6,*) 'The last component fraction will be calculated for you'
	  exit
    end if
    write(6,3000) i
    read(5,*) x_tmp(i)
    sum = sum + x_tmp(i)
  end do

  x_tmp(ncp) = 1.0d0 - sum 
  sum = 0.0d0

if (pick.eq.1) then
	write(6,*) '============================================================================'
	write(6,*) '=                                                                          ='
	write(6,*) '=   Please ensure that the temperature and composition pts. in             ='
	write(6,*) '=                  the file called T_X_inpt.txt                            ='
	write(6,*) '============================================================================'

	open(unit=12, file= "T_X_inpt.txt", action= "read") 
		write(6,*)'Enter the number of data points:'
		read(12,*) npts 	

		write(6,*)'Enter the temperatures (Kelvin) followed by compositions:'
  
		do i=1, npts
			read(12,*)TCA(i), (xcomp(i,j),j=1,ncp-1) 
		end do
	close( unit=12, status="keep")

! calculate the composition of the last component
  do i=1,npts
     sum=0.0d0
	 do j=1,ncp-1
		sum=sum+xcomp(i,j)
	 end do
		xcomp(i,ncp)=1.0d0-sum 
		if (sum.gt.1.0.or.sum.lt.0.0) then
		  write(6,*)'Wrong Compositions !!'
		  stop
		end if		 
  end do
  sum=0.0d0
   if (polymer.eq.2) then
	 do i=1,npts
	  sum=0.0d0
      do j = 1, ncp
	   sum = sum + xcomp(i,j) / mw_tmp(j)
      end do
      do j = 1, ncp
	   xcomp(i,j) = xcomp(i,j) / mw_tmp(j) / sum
      end do
     end do
   end if
end if


sum = 0.0d0
!	Converting mass to mole fractions
if (polymer.eq.2) then
	
    do i = 1, ncp
	  sum = sum + x_tmp(i) / mw_tmp(i)
    end do
    do i = 1, ncp
	  x_tmp(i) = x_tmp(i) / mw_tmp(i) / sum
    end do
end if

!	Input the interaction parameters Kij below
!  forall (i=1:ncp)
!    kij_tmp(i,i) = 0.0d0
!  end forall
!if (pick2.eq.2) then
!  write(6,*) ' '
! write(6,*) 'Enter the interaction parameters Kij below'
!  do i = 1, ncp
!    do j = i+1, ncp
! 	  write(6,*) 'For i =',i, 'j=',j
!	  read(5,*) kij_tmp(i,j)
!	  kij_tmp(j,i) = kij_tmp(i,j)
!   end do
!  end do
!end if

end if

if (pick.eq.2) then
  write(6,*) ' '
  write(6,*) '==========================================================================='
  write(6,*) '=                                                                         ='
  write(6,*) '=     Please specify whether you wish to:                                 ='
  write(6,*) '=            (1) enter the data points by hand or                         ='
  write(6,*) '=            (2) use a preformatted expdat.dat file.  The file should be  ='
  write(6,*) '=                  formatted as:                                          ='
  write(6,*) '=                                 # of exp. data points                   ='
  write(6,*) '=                                 T, P(bar), x(2), y(2) (for each point)  ='  
  write(6,*) '=                                 Note: component 2 should be #2 in input ='
  write(6,*) '==========================================================================='
  read(5,*) pick3

  if (pick3.eq.1) then
    write(6,*) 'Enter the number of data point:'
	read(5,*) ndp_exp
	write(6,*) 'For each data point, enter the following information'

	write(6,*) 'Experimental T(K), P(Bars), x2 (mole frac), y2(mole frac) '

	do i = 1, ndp_exp
	  read(5,*) TK_exp(i), Pset_exp(i), x_exp(2,i), y_exp(2,i)	  
	end do
  else 
    write(6,*) 'Reading the experimental info'
	open(unit=2, file='expdat.dat', form='formatted',status='unknown')
	rewind (2)
	read(2,*) ndp_exp

	do i = 1, ndp_exp  	  
	  read(2,*) TK_exp(i), Pset_exp(i), x_exp(2,i), y_exp(2,i)
    end do
  end if
end if

if (pick2.eq.3) then
!	!	Input the kappa and epsilon parameters below
!	if (max.gt.0) then
!	 write(6,*) ' '
!	 write(6,*) 'Enter the association epsilon (Well depth) interaction parameters below'
!
!	 do j = 1, ncp
!		 do i = 1, N_sites_tmp(j)
!		  do k = 1, ncp
!			do l = 1, N_sites_tmp(k)
!			 write(6,*) 'interaction between site',i,' on molecule', j,' and'
!			 write(6,*) 'site',l, ' on molecule', k
!			 read(5,*) fa_tmp(j,k,i,l,2)
!			end do
!		  end do
!	     end do
!	 end do

!	 write(6,*) ' '
!	 write(6,*) 'Enter the association kappa (Well width) interaction parameters below'
!	  do j = 1, ncp
!		do i = 1, N_sites_tmp(j)
 !		 do k = 1, ncp
!  			do l = 1, N_sites_tmp(k)
!			write(6,*) 'interaction between site',i,' on molecule', j, ' and'
! 			write(6,*) 'site',l, ' on molecule', k
!			read(5,*) kappa_tmp(j,k,i,l)
!			end do
!		 end do
!	 	end do
!	  end do
!	end if

end if
		
!	Below we will convert the input parameters and association parameters to internally
!	used units.

max = 0
do i = 1, ncp
  if (max .lt. N_sites_tmp(i)) then
    max = N_sites_tmp(i)
  end if
end do

forall (i=1:ncp)
  sig_tmp(i,i) = (v00_tmp(i)*6.0d0*tau/pi/AVOG)**(1.0d0/3.0d0)
end forall

forall (i=1:ncp, j=1:ncp, i .ne. j)
  sig_tmp(i,j) = 0.5d0 * (sig_tmp(i,i) + sig_tmp(j,j))
end forall

!	converting v00 from mL/mol to angstroms /molecule
forall (i=1:ncp)
  v00_tmp(i) = v00_tmp(i) / AVOG
end forall

if (pick.eq.2) then
! Problem with fb- it's T dependent. Allocate it here at the first temperature. 
! Will need to write an isobaric routine that re-calculates FB as F(temp) 
	TK=TK_exp(1)
end if


!	assigning association parameters
do j = 1, ncp
  do i = 1, N_sites_tmp(j)
 	do k = 1, ncp
  	  do l = 1, N_sites_tmp(k)
		fb_tmp(j,k,i,l,1) = fa_tmp(j,k,i,l,2)/TK/TK * exp(fa_tmp(j,k,i,l,2)/TK)
		fa_tmp(j,k,i,l,1) = exp(fa_tmp(j,k,i,l,2)/TK) - 1.0d0 			
  	  end do
	end do
  end do
end do

forall (i=1:ncp)
  z(i) = x_tmp(i)
end forall

END SUBROUTINE
